﻿using System;
using System.Runtime.InteropServices;
using TweetSharp;
using Interop.Plantronics;

namespace TweetMe
{
    public class TweetMe
    {
        private static COMDevice _device;
        static ICOMDeviceListenerEvents_Event _devListener;
        private static ICOMHostCommandExt _command;

        //Twitter integration stuff
        //NOTE: you must get your own keys/tokens in order to make this tweet to your _twitter account!
        //To do this simply register at the following site: https://dev.twitter.com/apps/new
        static String twitterConsumerKey = "yourConsumerKey";
        static String twitterConsumerSecret = "yourConsumerSecret";
        static String twitterAccessToken = "yourAccessToken";
        static String twitterAccessTokenSecret = "yourAccessTokenSecret";
        static String twitterUserHandle = "yourUserHandle";

        static TwitterService _twitter;
        

        static void Main()
        {
            //Get a valid _device
            COMSessionManager mgr = new COMSessionManager();
            COMSession sess;
            mgr.Register("My Session", out sess);
            _device = sess.GetActiveDevice();

            //Cast HostCommand and init _command.
            _command = _device.HostCommand as ICOMHostCommandExt;

            //Cast HostCommand and DeviceListener and init _command and _devListener.
            _command = _device.HostCommand as ICOMHostCommandExt;
            _devListener = _device.DeviceListener;
            
            InitializeTwitter();

            Menu();
            int choice = 0;
            while (choice != 4)
            {
                choice = Convert.ToInt32(Console.ReadLine());

                switch (choice)
                {
                    case 1:
                        RequestProximity();
                        break;
                    case 2:
                        if (_devListener != null)
                        {
                            Console.WriteLine("Listening to events.");
                            _devListener.onHeadsetStateChanged += devListener_HeadsetStateChanged;
                        }
                        break;
                    case 3:
                        if (_devListener != null)
                        {
                            Console.WriteLine("Stopped listening to events.");
                            _devListener.onHeadsetStateChanged -= devListener_HeadsetStateChanged;
                        }
                        break;
                    case 4:
                        break;
                    default:
                        Console.WriteLine("Please enter a value between 1 and 4. \r\n");
                        Menu();
                        break;
                }
            }
        }

        //_twitter initialization
        private static void InitializeTwitter()
        {
            //initialize _twitter
            _twitter = new TwitterService(twitterConsumerKey, twitterConsumerSecret);
            _twitter.AuthenticateWith(twitterAccessToken, twitterAccessTokenSecret);
        }

        //_twitter send tweet and display status
        private static void SendTweet(string text)
        {
            SendTweetOptions options = new SendTweetOptions {Status = text};
            TwitterStatus status = _twitter.SendTweet(options);
            if (status == null)
            {
                Console.WriteLine("Tweet failed, no status returned. (have you added key/token/user etc at top of code?)");
            }
            else
            {
                Console.WriteLine("Created a new tweet, id = " + status.Id);
            }
        }

        private static void Menu()
        {
            Console.WriteLine("Plantronics SDK Headset Proximity Sample\r\n" +
                              "\r\n" +
                              "1. Request headset proximity. \r\n" +
                              "2. Start listening for events. \r\n" +
                              "3. Stop listening for events. \r\n" +
                              "4. Exit application.");
        }

        private static void devListener_HeadsetStateChanged(COMDeviceListenerEventArgs args)
        {
            switch (args.HeadsetStateChange)
            {
                case (DeviceHeadsetStateChange.HeadsetStateChange_Don):
                    SendTweet("At " + DateTime.Now.ToString("H:mm:ss") + " my Plantronics headset sensors told me that " + twitterUserHandle + " is wearing me!");
                    break;
                case (DeviceHeadsetStateChange.HeadsetStateChange_Doff):
                    SendTweet("At " + DateTime.Now.ToString("H:mm:ss") + " my Plantronics headset sensors told me that " + twitterUserHandle + " has taken me off.");
                    break;
                case (DeviceHeadsetStateChange.HeadsetStateChange_Far):
                    SendTweet("At " + DateTime.Now.ToString("H:mm:ss") + " my Plantronics headset sensors told me that " + twitterUserHandle + " is away from their desk.");
                    break;
                case (DeviceHeadsetStateChange.HeadsetStateChange_Near):
                    SendTweet("At " + DateTime.Now.ToString("H:mm:ss") + " my Plantronics headset sensors told me that " + twitterUserHandle + " is at their desk.");
                    break;
                default:
                    SendTweet("At " + DateTime.Now.ToString("H:mm:ss") + " the following Plantronics headset event " + args.HeadsetStateChange + " occured for " + twitterUserHandle);
                    break;
            }
        }

        private static void RequestProximity()
        {
            try
            {
                if (_command != null)
                    _command.RequestProximity();
            }
            catch (COMException ex)
            {
                Console.WriteLine(ex.Message + "\r\nPlease use a _device that supports proximity.");
            }
        }
    }
}
